# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'FlowWindow.ui'
#
# Created: Fri May  7 17:20:33 2010
#      by: PyQt4 UI code generator 4.4.2
#
# WARNING! All changes made in this file will be lost!

try:
    from OVEStandard import globalForcePySide
    if globalForcePySide: raise Exception()
    from PyQt4 import QtCore, QtGui
except:
    from PySide import QtCore, QtGui

class Ui_FlowWindow(object):
    def setupUi(self, FlowWindow):
        FlowWindow.setObjectName("FlowWindow")
        FlowWindow.resize(800,600)
        self.centralwidget = QtGui.QWidget(FlowWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.gridLayout = QtGui.QGridLayout(self.centralwidget)
        self.gridLayout.setObjectName("gridLayout")
        self.tabWidget = QtGui.QTabWidget(self.centralwidget)
        self.tabWidget.setObjectName("tabWidget")
        self.unset = QtGui.QWidget()
        self.unset.setObjectName("unset")
        self.gridLayout_10 = QtGui.QGridLayout(self.unset)
        self.gridLayout_10.setObjectName("gridLayout_10")
        self.tabWidget.addTab(self.unset,"")
        self.gridLayout.addWidget(self.tabWidget,0,0,1,1)
        self.horizontalLayout_2 = QtGui.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.ssgCheckBox = QtGui.QCheckBox(self.centralwidget)
        self.ssgCheckBox.setObjectName("ssgCheckBox")
        self.horizontalLayout_2.addWidget(self.ssgCheckBox)
        self.ssgComboBox = QtGui.QComboBox(self.centralwidget)
        self.ssgComboBox.setEditable(True)
        self.ssgComboBox.setMaxVisibleItems(20)
        self.ssgComboBox.setInsertPolicy(QtGui.QComboBox.NoInsert)
        self.ssgComboBox.setMinimumContentsLength(32)
        self.ssgComboBox.setObjectName("ssgComboBox")
        self.horizontalLayout_2.addWidget(self.ssgComboBox)
        self.ssgSaveButton = QtGui.QPushButton(self.centralwidget)
        self.ssgSaveButton.setObjectName("ssgSaveButton")
        self.horizontalLayout_2.addWidget(self.ssgSaveButton)
        self.ssgDeleteButton = QtGui.QPushButton(self.centralwidget)
        self.ssgDeleteButton.setObjectName("ssgDeleteButton")
        self.horizontalLayout_2.addWidget(self.ssgDeleteButton)
        spacerItem = QtGui.QSpacerItem(40,20,QtGui.QSizePolicy.Expanding,QtGui.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem)
        self.gridLayout.addLayout(self.horizontalLayout_2,1,0,1,1)
        self.horizontalLayout = QtGui.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.hostLabel = QtGui.QLabel(self.centralwidget)
        self.hostLabel.setObjectName("hostLabel")
        self.horizontalLayout.addWidget(self.hostLabel)
        self.hostComboBox = QtGui.QComboBox(self.centralwidget)
        self.hostComboBox.setSizeAdjustPolicy(QtGui.QComboBox.AdjustToContents)
        self.hostComboBox.setObjectName("hostComboBox")
        self.horizontalLayout.addWidget(self.hostComboBox)
        self.intervalCheckBox = QtGui.QCheckBox(self.centralwidget)
        self.intervalCheckBox.setObjectName("intervalCheckBox")
        self.horizontalLayout.addWidget(self.intervalCheckBox)
        self.intervalSpinBox = QtGui.QSpinBox(self.centralwidget)
        self.intervalSpinBox.setMinimum(1)
        self.intervalSpinBox.setMaximum(1000000)
        self.intervalSpinBox.setObjectName("intervalSpinBox")
        self.horizontalLayout.addWidget(self.intervalSpinBox)
        spacerItem1 = QtGui.QSpacerItem(40,20,QtGui.QSizePolicy.Expanding,QtGui.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)
        self.fetchPathsButton = QtGui.QPushButton(self.centralwidget)
        self.fetchPathsButton.setObjectName("fetchPathsButton")
        self.horizontalLayout.addWidget(self.fetchPathsButton)
        self.fetchButton = QtGui.QPushButton(self.centralwidget)
        self.fetchButton.setObjectName("fetchButton")
        self.horizontalLayout.addWidget(self.fetchButton)
        self.gridLayout.addLayout(self.horizontalLayout,3,0,1,1)
        self.line = QtGui.QFrame(self.centralwidget)
        self.line.setFrameShape(QtGui.QFrame.HLine)
        self.line.setFrameShadow(QtGui.QFrame.Sunken)
        self.line.setObjectName("line")
        self.gridLayout.addWidget(self.line,2,0,1,1)
        FlowWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtGui.QMenuBar(FlowWindow)
        self.menubar.setGeometry(QtCore.QRect(0,0,800,28))
        self.menubar.setObjectName("menubar")
        self.menuFile = QtGui.QMenu(self.menubar)
        self.menuFile.setObjectName("menuFile")
        FlowWindow.setMenuBar(self.menubar)
        self.statusbar = QtGui.QStatusBar(FlowWindow)
        self.statusbar.setObjectName("statusbar")
        FlowWindow.setStatusBar(self.statusbar)
        self.actionShow_Log = QtGui.QAction(FlowWindow)
        self.actionShow_Log.setObjectName("actionShow_Log")
        self.actionNew_DB_Window = QtGui.QAction(FlowWindow)
        self.actionNew_DB_Window.setObjectName("actionNew_DB_Window")
        self.actionPreferences = QtGui.QAction(FlowWindow)
        self.actionPreferences.setObjectName("actionPreferences")
        self.actionQuit = QtGui.QAction(FlowWindow)
        self.actionQuit.setObjectName("actionQuit")
        self.actionNew_Flow_Window = QtGui.QAction(FlowWindow)
        self.actionNew_Flow_Window.setObjectName("actionNew_Flow_Window")
        self.menuFile.addAction(self.actionNew_DB_Window)
        self.menuFile.addAction(self.actionNew_Flow_Window)
        self.menuFile.addAction(self.actionShow_Log)
        self.menuFile.addAction(self.actionPreferences)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.actionQuit)
        self.menubar.addAction(self.menuFile.menuAction())
        self.hostLabel.setBuddy(self.hostComboBox)

        self.retranslateUi(FlowWindow)
        self.tabWidget.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(FlowWindow)

    def retranslateUi(self, FlowWindow):
        FlowWindow.setWindowTitle(QtGui.QApplication.translate("FlowWindow", "OVSDB Monitor", None, QtGui.QApplication.UnicodeUTF8))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.unset), QtGui.QApplication.translate("FlowWindow", "Awaiting update...", None, QtGui.QApplication.UnicodeUTF8))
        self.ssgCheckBox.setText(QtGui.QApplication.translate("FlowWindow", "Server-side grep", None, QtGui.QApplication.UnicodeUTF8))
        self.ssgSaveButton.setText(QtGui.QApplication.translate("FlowWindow", "Save", None, QtGui.QApplication.UnicodeUTF8))
        self.ssgDeleteButton.setText(QtGui.QApplication.translate("FlowWindow", "Delete", None, QtGui.QApplication.UnicodeUTF8))
        self.hostLabel.setText(QtGui.QApplication.translate("FlowWindow", "Host", None, QtGui.QApplication.UnicodeUTF8))
        self.intervalCheckBox.setText(QtGui.QApplication.translate("FlowWindow", "Auto-refetch every", None, QtGui.QApplication.UnicodeUTF8))
        self.intervalSpinBox.setSuffix(QtGui.QApplication.translate("FlowWindow", "s", None, QtGui.QApplication.UnicodeUTF8))
        self.fetchPathsButton.setToolTip(QtGui.QApplication.translate("FlowWindow", "Refetches the datapath names and rebuilds the window tabs to reflect them.  Use when the network has been reconfigured, e.g. a bond has been created", None, QtGui.QApplication.UnicodeUTF8))
        self.fetchPathsButton.setText(QtGui.QApplication.translate("FlowWindow", "Refetch Datapath List", None, QtGui.QApplication.UnicodeUTF8))
        self.fetchButton.setText(QtGui.QApplication.translate("FlowWindow", "Refetch", None, QtGui.QApplication.UnicodeUTF8))
        self.menuFile.setTitle(QtGui.QApplication.translate("FlowWindow", "File", None, QtGui.QApplication.UnicodeUTF8))
        self.actionShow_Log.setText(QtGui.QApplication.translate("FlowWindow", "Show Log", None, QtGui.QApplication.UnicodeUTF8))
        self.actionNew_DB_Window.setText(QtGui.QApplication.translate("FlowWindow", "New DB Window", None, QtGui.QApplication.UnicodeUTF8))
        self.actionPreferences.setText(QtGui.QApplication.translate("FlowWindow", "Preferences", None, QtGui.QApplication.UnicodeUTF8))
        self.actionQuit.setText(QtGui.QApplication.translate("FlowWindow", "Quit", None, QtGui.QApplication.UnicodeUTF8))
        self.actionNew_Flow_Window.setText(QtGui.QApplication.translate("FlowWindow", "New Flow Window", None, QtGui.QApplication.UnicodeUTF8))

