# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '../ovsdb/ovsdbmonitor/MainWindow.ui'
#
# Created: Mon May 17 16:23:47 2010
#      by: PyQt4 UI code generator 4.7.3
#
# WARNING! All changes made in this file will be lost!


try:
    from OVEStandard import globalForcePySide
    if globalForcePySide:
        raise Exception()
    from PyQt4 import QtCore, QtGui
except:
    from PySide import QtCore, QtGui

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(800, 600)
        self.centralwidget = QtGui.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.gridLayout = QtGui.QGridLayout(self.centralwidget)
        self.gridLayout.setObjectName("gridLayout")
        self.verticalLayout = QtGui.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.tabWidget = QtGui.QTabWidget(self.centralwidget)
        self.tabWidget.setObjectName("tabWidget")
        self.Bridge = QtGui.QWidget()
        self.Bridge.setObjectName("Bridge")
        self.gridLayout_2 = QtGui.QGridLayout(self.Bridge)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.BridgeTable = QtGui.QTableWidget(self.Bridge)
        self.BridgeTable.setObjectName("BridgeTable")
        self.BridgeTable.setColumnCount(0)
        self.BridgeTable.setRowCount(0)
        self.gridLayout_2.addWidget(self.BridgeTable, 0, 0, 1, 1)
        self.tabWidget.addTab(self.Bridge, "")
        self.Controller = QtGui.QWidget()
        self.Controller.setObjectName("Controller")
        self.gridLayout_3 = QtGui.QGridLayout(self.Controller)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.ControllerTable = QtGui.QTableWidget(self.Controller)
        self.ControllerTable.setObjectName("ControllerTable")
        self.ControllerTable.setColumnCount(0)
        self.ControllerTable.setRowCount(0)
        self.gridLayout_3.addWidget(self.ControllerTable, 0, 0, 1, 1)
        self.tabWidget.addTab(self.Controller, "")
        self.Interface = QtGui.QWidget()
        self.Interface.setObjectName("Interface")
        self.gridLayout_4 = QtGui.QGridLayout(self.Interface)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.InterfaceTable = QtGui.QTableWidget(self.Interface)
        self.InterfaceTable.setObjectName("InterfaceTable")
        self.InterfaceTable.setColumnCount(0)
        self.InterfaceTable.setRowCount(0)
        self.gridLayout_4.addWidget(self.InterfaceTable, 0, 0, 1, 1)
        self.tabWidget.addTab(self.Interface, "")
        self.Mirror = QtGui.QWidget()
        self.Mirror.setObjectName("Mirror")
        self.gridLayout_5 = QtGui.QGridLayout(self.Mirror)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.MirrorTable = QtGui.QTableWidget(self.Mirror)
        self.MirrorTable.setObjectName("MirrorTable")
        self.MirrorTable.setColumnCount(0)
        self.MirrorTable.setRowCount(0)
        self.gridLayout_5.addWidget(self.MirrorTable, 0, 0, 1, 1)
        self.tabWidget.addTab(self.Mirror, "")
        self.NetFlow = QtGui.QWidget()
        self.NetFlow.setObjectName("NetFlow")
        self.gridLayout_6 = QtGui.QGridLayout(self.NetFlow)
        self.gridLayout_6.setObjectName("gridLayout_6")
        self.NetFlowTable = QtGui.QTableWidget(self.NetFlow)
        self.NetFlowTable.setObjectName("NetFlowTable")
        self.NetFlowTable.setColumnCount(0)
        self.NetFlowTable.setRowCount(0)
        self.gridLayout_6.addWidget(self.NetFlowTable, 0, 0, 1, 1)
        self.tabWidget.addTab(self.NetFlow, "")
        self.Open_vSwitch = QtGui.QWidget()
        self.Open_vSwitch.setObjectName("Open_vSwitch")
        self.gridLayout_7 = QtGui.QGridLayout(self.Open_vSwitch)
        self.gridLayout_7.setObjectName("gridLayout_7")
        self.Open_vSwitchTable = QtGui.QTableWidget(self.Open_vSwitch)
        self.Open_vSwitchTable.setObjectName("Open_vSwitchTable")
        self.Open_vSwitchTable.setColumnCount(0)
        self.Open_vSwitchTable.setRowCount(0)
        self.gridLayout_7.addWidget(self.Open_vSwitchTable, 0, 0, 1, 1)
        self.tabWidget.addTab(self.Open_vSwitch, "")
        self.Port = QtGui.QWidget()
        self.Port.setObjectName("Port")
        self.gridLayout_8 = QtGui.QGridLayout(self.Port)
        self.gridLayout_8.setObjectName("gridLayout_8")
        self.PortTable = QtGui.QTableWidget(self.Port)
        self.PortTable.setObjectName("PortTable")
        self.PortTable.setColumnCount(0)
        self.PortTable.setRowCount(0)
        self.gridLayout_8.addWidget(self.PortTable, 0, 0, 1, 1)
        self.tabWidget.addTab(self.Port, "")
        self.QoS = QtGui.QWidget()
        self.QoS.setObjectName("QoS")
        self.gridLayout_10 = QtGui.QGridLayout(self.QoS)
        self.gridLayout_10.setObjectName("gridLayout_10")
        self.QoSTable = QtGui.QTableWidget(self.QoS)
        self.QoSTable.setObjectName("QoSTable")
        self.QoSTable.setColumnCount(0)
        self.QoSTable.setRowCount(0)
        self.gridLayout_10.addWidget(self.QoSTable, 0, 0, 1, 1)
        self.tabWidget.addTab(self.QoS, "")
        self.Queue = QtGui.QWidget()
        self.Queue.setObjectName("Queue")
        self.gridLayout_11 = QtGui.QGridLayout(self.Queue)
        self.gridLayout_11.setObjectName("gridLayout_11")
        self.QueueTable = QtGui.QTableWidget(self.Queue)
        self.QueueTable.setObjectName("QueueTable")
        self.QueueTable.setColumnCount(0)
        self.QueueTable.setRowCount(0)
        self.gridLayout_11.addWidget(self.QueueTable, 0, 0, 1, 1)
        self.tabWidget.addTab(self.Queue, "")
        self.sFlow = QtGui.QWidget()
        self.sFlow.setObjectName("sFlow")
        self.gridLayout_9 = QtGui.QGridLayout(self.sFlow)
        self.gridLayout_9.setObjectName("gridLayout_9")
        self.sFlowTable = QtGui.QTableWidget(self.sFlow)
        self.sFlowTable.setObjectName("sFlowTable")
        self.sFlowTable.setColumnCount(0)
        self.sFlowTable.setRowCount(0)
        self.gridLayout_9.addWidget(self.sFlowTable, 0, 0, 1, 1)
        self.tabWidget.addTab(self.sFlow, "")
        self.SSL = QtGui.QWidget()
        self.SSL.setObjectName("SSL")
        self.gridLayout_101 = QtGui.QGridLayout(self.SSL)
        self.gridLayout_101.setObjectName("gridLayout_101")
        self.SSLTable = QtGui.QTableWidget(self.SSL)
        self.SSLTable.setObjectName("SSLTable")
        self.SSLTable.setColumnCount(0)
        self.SSLTable.setRowCount(0)
        self.gridLayout_101.addWidget(self.SSLTable, 0, 0, 1, 1)
        self.tabWidget.addTab(self.SSL, "")
        self.verticalLayout.addWidget(self.tabWidget)
        self.horizontalLayout = QtGui.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.hostLabel = QtGui.QLabel(self.centralwidget)
        self.hostLabel.setObjectName("hostLabel")
        self.horizontalLayout.addWidget(self.hostLabel)
        self.hostComboBox = QtGui.QComboBox(self.centralwidget)
        self.hostComboBox.setSizeAdjustPolicy(QtGui.QComboBox.AdjustToContents)
        self.hostComboBox.setObjectName("hostComboBox")
        self.horizontalLayout.addWidget(self.hostComboBox)
        self.intervalCheckBox = QtGui.QCheckBox(self.centralwidget)
        self.intervalCheckBox.setObjectName("intervalCheckBox")
        self.horizontalLayout.addWidget(self.intervalCheckBox)
        self.intervalSpinBox = QtGui.QSpinBox(self.centralwidget)
        self.intervalSpinBox.setMinimum(1)
        self.intervalSpinBox.setMaximum(1000000)
        self.intervalSpinBox.setObjectName("intervalSpinBox")
        self.horizontalLayout.addWidget(self.intervalSpinBox)
        spacerItem = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.fetchButton = QtGui.QPushButton(self.centralwidget)
        self.fetchButton.setObjectName("fetchButton")
        self.horizontalLayout.addWidget(self.fetchButton)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.gridLayout.addLayout(self.verticalLayout, 0, 0, 1, 1)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtGui.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 800, 28))
        self.menubar.setObjectName("menubar")
        self.menuFile = QtGui.QMenu(self.menubar)
        self.menuFile.setObjectName("menuFile")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtGui.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)
        self.actionShow_Log = QtGui.QAction(MainWindow)
        self.actionShow_Log.setObjectName("actionShow_Log")
        self.actionNew_DB_Window = QtGui.QAction(MainWindow)
        self.actionNew_DB_Window.setObjectName("actionNew_DB_Window")
        self.actionPreferences = QtGui.QAction(MainWindow)
        self.actionPreferences.setObjectName("actionPreferences")
        self.actionQuit = QtGui.QAction(MainWindow)
        self.actionQuit.setObjectName("actionQuit")
        self.actionNew_Flow_Window = QtGui.QAction(MainWindow)
        self.actionNew_Flow_Window.setObjectName("actionNew_Flow_Window")
        self.menuFile.addAction(self.actionNew_DB_Window)
        self.menuFile.addAction(self.actionNew_Flow_Window)
        self.menuFile.addAction(self.actionShow_Log)
        self.menuFile.addAction(self.actionPreferences)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.actionQuit)
        self.menubar.addAction(self.menuFile.menuAction())
        self.hostLabel.setBuddy(self.hostComboBox)

        self.retranslateUi(MainWindow)
        self.tabWidget.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(QtGui.QApplication.translate("MainWindow", "OVSDB Monitor", None, QtGui.QApplication.UnicodeUTF8))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.Bridge), QtGui.QApplication.translate("MainWindow", "Bridge", None, QtGui.QApplication.UnicodeUTF8))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.Controller), QtGui.QApplication.translate("MainWindow", "Controller", None, QtGui.QApplication.UnicodeUTF8))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.Interface), QtGui.QApplication.translate("MainWindow", "Interface", None, QtGui.QApplication.UnicodeUTF8))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.Mirror), QtGui.QApplication.translate("MainWindow", "Mirror", None, QtGui.QApplication.UnicodeUTF8))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.NetFlow), QtGui.QApplication.translate("MainWindow", "NetFlow", None, QtGui.QApplication.UnicodeUTF8))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.Open_vSwitch), QtGui.QApplication.translate("MainWindow", "Open_vSwitch", None, QtGui.QApplication.UnicodeUTF8))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.Port), QtGui.QApplication.translate("MainWindow", "Port", None, QtGui.QApplication.UnicodeUTF8))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.QoS), QtGui.QApplication.translate("MainWindow", "QoS", None, QtGui.QApplication.UnicodeUTF8))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.Queue), QtGui.QApplication.translate("MainWindow", "Queue", None, QtGui.QApplication.UnicodeUTF8))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.sFlow), QtGui.QApplication.translate("MainWindow", "sFlow", None, QtGui.QApplication.UnicodeUTF8))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.SSL), QtGui.QApplication.translate("MainWindow", "SSL", None, QtGui.QApplication.UnicodeUTF8))
        self.hostLabel.setText(QtGui.QApplication.translate("MainWindow", "Host", None, QtGui.QApplication.UnicodeUTF8))
        self.intervalCheckBox.setText(QtGui.QApplication.translate("MainWindow", "Auto-refetch every", None, QtGui.QApplication.UnicodeUTF8))
        self.intervalSpinBox.setSuffix(QtGui.QApplication.translate("MainWindow", "s", None, QtGui.QApplication.UnicodeUTF8))
        self.fetchButton.setText(QtGui.QApplication.translate("MainWindow", "Refetch", None, QtGui.QApplication.UnicodeUTF8))
        self.menuFile.setTitle(QtGui.QApplication.translate("MainWindow", "File", None, QtGui.QApplication.UnicodeUTF8))
        self.actionShow_Log.setText(QtGui.QApplication.translate("MainWindow", "Show Log", None, QtGui.QApplication.UnicodeUTF8))
        self.actionNew_DB_Window.setText(QtGui.QApplication.translate("MainWindow", "New DB Window", None, QtGui.QApplication.UnicodeUTF8))
        self.actionPreferences.setText(QtGui.QApplication.translate("MainWindow", "Preferences", None, QtGui.QApplication.UnicodeUTF8))
        self.actionQuit.setText(QtGui.QApplication.translate("MainWindow", "Quit", None, QtGui.QApplication.UnicodeUTF8))
        self.actionNew_Flow_Window.setText(QtGui.QApplication.translate("MainWindow", "New Flow Window", None, QtGui.QApplication.UnicodeUTF8))

